import os
import pickle
import pkg_resources

dir_path = os.path.dirname(os.path.realpath(__file__))

data = {}
with open(os.path.join(dir_path, 'dict_antonym.txt'), 'rt') as fd:
    for line in fd:
        if not line:
            continue
        source, target = line.strip().split(':')
        data[source] = target
        data[target] = source

output_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'antonym.pkl')
with open(output_file, 'wb') as fd:
    pickle.dump(data, fd)


class Antonym(object):
    def __init__(self):
        data_file = pkg_resources.resource_filename(__name__, "antonym.pkl")
        with open(data_file, 'rb') as fd:
            self.data = pickle.load(fd)

    def query(self, input_char):
        result = self.data.get(input_char, input_char)
        return result



# -*- coding:utf-8 -*-
from collections import defaultdict
import re
import os
import pickle
import pkg_resources

__all__ = ['NaiveFilter', 'BSFilter', 'DFAFilter']
__author__ = 'observer'
__date__ = '2012.01.05'

source_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'keywords.txt')
output_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'keywords.pkl')

data = []
with open(source_path, 'rt') as f:
    for word in f:
        data.append(word.strip().lower())

with open(output_file, 'wb') as fd:
    pickle.dump(data, fd)


class BaseFilter(object):
    '''Filter Messages from keywords

    Use Back Sorted Mapping to reduce replacement times

    >>> f = BaseFilter()
    >>> f.add("sexy")
    >>> f.filter("hello sexy baby")
    hello **** baby
    '''
    def __init__(self):
        data_file = pkg_resources.resource_filename(__name__, 'keywords.pkl')
        with open(data_file, 'rb') as f:
            self.keywords = pickle.load(f)

    def add(self, keyword):
        keyword = keyword.lower()
        if keyword not in self.keywords:
            self.keywords.add(keyword)

    def filter(self, message, repl="*"):
        message = message.lower()
        for kw in self.keywords:
            message = message.replace(kw, repl * len(kw))
        return message


class NaiveFilter(BaseFilter):
    pass


class BSFilter(BaseFilter):

    def __init__(self):
        super(BSFilter, self).__init__()
        self.kwsets = set(self.keywords)
        self.pat_en = re.compile(r'^[0-9a-zA-Z]+$')  # english phrase or not
        self.bsdict = defaultdict(set)

        for index, keyword in enumerate(self.keywords):
            for word in keyword.split():
                if self.pat_en.search(word):
                    self.bsdict[word].add(index)
                    continue
                for char in word:
                    self.bsdict[char].add(index)

    def add(self, keyword):
        keyword = keyword.lower()
        if keyword in self.kwsets:
            return
        self.keywords.append(keyword)
        self.kwsets.add(keyword)
        index = len(self.keywords) - 1
        for word in keyword.split():
            if self.pat_en.search(word):
                self.bsdict[word].add(index)
                continue
            for char in word:
                self.bsdict[char].add(index)

    def filter(self, message, repl="*"):
        message = message.lower()
        for word in message.split():
            if self.pat_en.search(word):
                for index in self.bsdict[word]:
                    message = message.replace(self.keywords[index], repl * len(self.keywords[index]))
                continue
            for char in word:
                for index in self.bsdict[char]:
                    message = message.replace(self.keywords[index], repl * len(self.keywords[index]))
        return message


class DFAFilter(BaseFilter):

    def __init__(self):
        super(DFAFilter, self).__init__()
        self.keyword_chains = {}
        self.delimit = '\x00'
        for keyword in self.keywords:
            self.add(keyword)

    def add(self, keyword):
        keyword = keyword.lower()
        chars = keyword.strip()
        if not chars:
            return
        level = self.keyword_chains
        i = 0
        for i in range(len(chars)):
            if chars[i] not in level:
                if not isinstance(level, dict):
                    break
                last_level = level
                last_char = chars[-1]
                for j in range(i, len(chars)):
                    level[chars[j]] = {}
                    last_level, last_char = level, chars[j]
                    level = level[chars[j]]
                last_level[last_char] = {self.delimit: 0}
                break
            level = level[chars[i]]
        if i == len(chars) - 1:
            level[self.delimit] = 0

    def filter(self, message, repl="*"):
        message = message.lower()
        ret = []
        start = 0
        while start < len(message):
            level = self.keyword_chains
            step_ins = 0
            for char in message[start:]:
                if char not in level:
                    ret.append(message[start])
                    break
                step_ins += 1
                if self.delimit in level[char]:
                    ret.append(repl * step_ins)
                    start += step_ins - 1
                    break
                level = level[char]
            else:
                ret.append(message[start])
            start += 1
        return ''.join(ret)



import os
import pickle
import pkg_resources

dir_path = os.path.dirname(os.path.realpath(__file__))

data = {}
for file_name in ['chaizi-ft.txt', 'chaizi-jt.txt']:
    with open(os.path.join(dir_path, file_name), 'rt') as fd:
        for line in fd:
            item_list = line.strip().split('\t')
            key = item_list[0]
            value = [i.strip().split() for i in item_list[1:]]
            data[key] = value

output_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'chaizi.pkl')
with open(output_file, 'wb') as fd:
    pickle.dump(data, fd)


class Chaizi(object):
    def __init__(self):
        data_file = pkg_resources.resource_filename(__name__, "chaizi.pkl")
        with open(data_file, 'rb') as fd:
            self.data = pickle.load(fd)

    def query(self, input_char):
        result = self.data.get(input_char, input_char)
        return result[0]


if __name__ == "__main__":
    hc = Chaizi()
    result = hc.query('名')
    print(result)




import jieba
from pypinyin import pinyin, Style

from .utils.textfilter.filter import DFAFilter
from .utils.chaizi.chaizi import Chaizi
from .utils.antonym.antonym import Antonym
from .utils.synonym.synonym import Synonym
from .utils.homophone.homophone import Homophone

jieba.enable_paddle()


class SayItAnotherWay(object):

    def __init__(self, sentence):
        self.sentence = sentence
        self.words = []

    def __str__(self):
        return self.sentence

    def translate(self, way, *args, **kwargs):
        if not hasattr(self, f'{way}_translate'):
            raise Exception('way error!')
        if not self.words or kwargs.get('refresh', False):
            self.refresh(*args, **kwargs)
        return getattr(self, f'{way}_translate')(*args, **kwargs)

    def refresh(self, *args, **kwargs):
        self.cut(*args, **kwargs)

    def mixed_translate(self, *args, **kwargs):
        words = kwargs.pop('words', self.words)
        ways = kwargs.pop('ways', [])
        if not ways:
            return ''.join(words)
        for way in ways:
            if not hasattr(self, f'_{way}_translate'):
                raise Exception('way error!')
            words = getattr(self, f'_{way}_translate')(words, *args, **kwargs)
        return ''.join(words)

    def pinyin_translate(self, *args, **kwargs):
        """拼音、首字母"""
        words = kwargs.pop('words', self.words)
        space_mark = '' if 'style' in kwargs else ' '
        style = getattr(Style, kwargs.pop('style', 'TONE'))
        return space_mark.join(self._pinyin_translate(words, *args, style=style, space_mark=space_mark, **kwargs))

    def _pinyin_translate(self, words, *args, style=Style.TONE, space_mark=' ', **kwargs):
        result = pinyin(words, style=style)
        return [space_mark.join(item) for item in result]

    def sequence_translate(self, *args, **kwargs):
        """打乱顺序"""
        words = kwargs.pop('words', self.words)
        result = self._sequence_translate(words, *args, **kwargs)
        return ''.join(result)

    def _sequence_translate(self, words, *args, **kwargs):
        result = [words[0]]
        total_num = len(words)
        for i in range(1, total_num, 2):
            if i + 1 != total_num:
                result.append(words[i + 1][::-1])
            result.append(words[i][::-1])
        return result

    def reverse_translate(self, *args, **kwargs):
        """逆序"""
        words = kwargs.pop('words', self.words)
        return ''.join(self._reverse_translate(words, *args, **kwargs))

    def _reverse_translate(self, words, *args, **kwargs):
        return [word[::-1] for word in words[::-1]]

    def filter_translate(self, *args, **kwargs):
        """过滤敏感词"""
        words = kwargs.pop('words', self.words)
        return ''.join(self._filter_translate(words, *args, **kwargs))

    def _filter_translate(self, words, *args, replace_str='*', **kwargs):
        gfw = DFAFilter()
        return [gfw.filter(word, replace_str) for word in words]

    def space_mark_translate(self, *args, **kwargs):
        """添加间隔符"""
        words = kwargs.pop('words', self.words)
        space_mark = kwargs.pop('space_mark', '-')
        return ''.join(self._space_mark_translate(words, space_mark, *args, **kwargs))

    def _space_mark_translate(self, words, space_mark, *args, **kwargs):
        result = []
        for word in words:
            result.append(word)
            result.append(space_mark)
        return result[:-1]

    def chaizi_translate(self, *args, **kwargs):
        """拆字"""
        words = kwargs.pop('words', self.words)
        return ' '.join(self._chaizi_translate(words, *args, **kwargs))

    def _chaizi_translate(self, words, *args, **kwargs):
        chaizi = Chaizi()
        return [''.join(chaizi.query(char)) for word in words for char in word]

    def antonym_translate(self, *args, **kwargs):
        """反义"""
        words = kwargs.pop('words', self.words)
        antonym = Antonym()
        return ''.join(self._antonym_translate(words, *args, **kwargs))

    def _antonym_translate(self, words, *args, **kwargs):
        antonym = Antonym()
        return [''.join(antonym.query(word)) for word in words]

    def synonym_translate(self, *args, **kwargs):
        """同义"""
        words = kwargs.pop('words', self.words)
        return ''.join(self._synonym_translate(words, *args, **kwargs))

    def _synonym_translate(self, words, *args, **kwargs):
        synonym = Synonym()
        return [''.join(synonym.query(word)) for word in words]

    def homophone_translate(self, *args, **kwargs):
        """同音"""
        words = kwargs.pop('words', self.words)
        return ''.join(self._homophone_translate(words, *args, **kwargs))

    def _homophone_translate(self, words, *args, **kwargs):
        homophone = Homophone()
        return ''.join(''.join(homophone.query(word)) for word in words)

    def cut(self, *args, **kwargs):
        if kwargs.get('use_jieba', True):
            self.words = self._jieba_cut(cut_all=kwargs.get('cut_all', False))
        else:
            self.words = self._normal_cout()

    def _jieba_cut(self, cut_all=False):
        return list(jieba.cut(self.sentence, cut_all=cut_all))

    def _normal_cout(self):
        return list(self.sentence)



# -*- coding: utf-8 -*-
import unittest

from siaw import SayItAnotherWay


class TestSayItAnotherWay(unittest.TestCase):

    def setUp(self):
        self.sentence_pinyin = SayItAnotherWay('阿伟死了')
        self.sentence_sequence = SayItAnotherWay('汉字的顺序并不一定能影响阅读')
        self.sentence_reverse = SayItAnotherWay('汉字的顺序并不一定能影响阅读')
        self.sentence_filter = SayItAnotherWay('违禁词会被过滤，比如巨乳，懂了吗？')
        self.sentence_space_mark = SayItAnotherWay('这里会加入一些间隔符号。')
        self.sentence_chaizi = SayItAnotherWay('测试拆字结果。')
        self.sentence_antonym = SayItAnotherWay('测试反义结果。')
        self.sentence_synonym = SayItAnotherWay('测试同义结果。')
        self.sentence_homophone = SayItAnotherWay('测试同音结果。')
        self.sentence_mixed = SayItAnotherWay('测试多种模式混合结果。')

    def test_pinyin_translate(self):
        """测试拼音转换"""
        self.assertEqual(self.sentence_pinyin.translate('pinyin'), 'ā wěi sǐ le')
        self.assertEqual(self.sentence_pinyin.translate('pinyin', style='FIRST_LETTER'), 'awsl')

    def test_sequence_translate(self):
        """测试顺序转换"""
        self.assertEqual(self.sentence_sequence.translate('sequence', use_jieba=False), '汉的字序顺不并定一影能阅响读')

    def test_reverse_translate(self):
        """测试逆序转换"""
        self.assertEqual(self.sentence_reverse.translate('reverse', use_jieba=False), '读阅响影能定一不并序顺的字汉')

    def test_filter_translate(self):
        """测试过滤敏感词转换"""
        self.assertEqual(self.sentence_filter.translate('filter'), '违禁词会被过滤，比如**，懂了吗？')

    def test_space_mark_translate(self):
        """测试添加间隔符转换"""
        self.assertEqual(self.sentence_space_mark.translate('space_mark', use_jieba=False, space_mark='+'),
                         '这+里+会+加+入+一+些+间+隔+符+号+。')

    def test_chaizi_translate(self):
        """测试拆字"""
        self.assertEqual(self.sentence_chaizi.translate('chaizi', use_jieba=False), '水则 言式 手斥 宀子 丝吉 日木 。')

    def test_antonym_translate(self):
        """测试反义"""
        self.assertNotEqual(self.sentence_antonym.translate('antonym'), '测试反义结果。')

    def test_synonym_translate(self):
        """测试同义"""
        self.assertNotEqual(self.sentence_synonym.translate('synonym'), '测试同义结果。')

    def test_homophone_translate(self):
        """测试同音"""
        self.assertNotEqual(self.sentence_homophone.translate('homophone'), '测试同音结果。')

    def test_mixed_translate(self):
        """测试混合模式"""
        self.assertEqual(self.sentence_mixed.translate('mixed', use_jieba=False, space_mark=' ', ways=['chaizi', 'pinyin']),
                         'shuǐzéyánshìxīxīhézhōngmùmòyìgōngshuǐkūnrényīkǒusījírìmù。')



import os
import random
import pickle
import pkg_resources

dir_path = os.path.dirname(os.path.realpath(__file__))

data = {'code2words': {}, 'word2code': {}}
with open(os.path.join(dir_path, 'dict_synonym.txt'), 'rt') as fd:
    for line in fd:
        if not line:
            continue
        split_char = ''
        for char in ['@', '=', '#']:
            if char in line:
                split_char = char
                break
        if not split_char:
            continue
        code, words = line.strip().split(split_char)
        words = words.strip('').split(' ')
        data['code2words'][code] = words
        for word in words:
            data['word2code'][word] = code

output_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'synonym.pkl')
with open(output_file, 'wb') as fd:
    pickle.dump(data, fd)


class Synonym(object):
    def __init__(self):
        data_file = pkg_resources.resource_filename(__name__, "synonym.pkl")
        with open(data_file, 'rb') as fd:
            self.data = pickle.load(fd)

    def query(self, input_char):
        synonym_code = self.data['word2code'].get(input_char, '')
        if not synonym_code:
            return input_char
        synonym_words = self.data['code2words'].get(synonym_code, [])
        synonym_words = [word for word in synonym_words if word and word != input_char]
        if not synonym_words:
            return input_char
        return synonym_words[random.randint(0, len(synonym_words) - 1)]



import os
import random
import pickle
import pkg_resources
from pypinyin import pinyin, Style

dir_path = os.path.dirname(os.path.realpath(__file__))

data = {}
for file_name in ['chinese_homophone_char.txt', 'chinese_homophone_word.txt']:
    with open(os.path.join(dir_path, file_name), 'rt') as fd:
        for line in fd:
            if not line:
                continue
            words = line.strip().split('\t')
            code = words.pop(0)
            code = code.replace('_', ' ')
            data[code] = words

output_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'homophone.pkl')
with open(output_file, 'wb') as fd:
    pickle.dump(data, fd)


class Homophone(object):
    def __init__(self):
        data_file = pkg_resources.resource_filename(__name__, "homophone.pkl")
        with open(data_file, 'rb') as fd:
            self.data = pickle.load(fd)

    def query(self, input_char):
        chars_pinyins = pinyin([input_char], style=Style.NORMAL)
        chars_pinyin = ' '.join(x[0] for x in chars_pinyins)
        homophone_words = self.data.get(chars_pinyin)
        if homophone_words:
            return self._filter(input_char, homophone_words)

        if chars_pinyins[0][0] == input_char:
            return input_char

        result = []
        for index, char_pinyin in enumerate(chars_pinyins):
            char = input_char[index]
            homophone_words = self.data.get(char_pinyin[0])
            result.append(self._filter(char, homophone_words) if homophone_words else char)
        return ''.join(result)

    @staticmethod
    def _filter(source_word, words):
        words = [word for word in words if word and word != source_word]
        if not words:
            return source_word
        return words[random.randint(0, len(words) - 1)]



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="siaw",
    version="0.0.1",
    packages=find_packages(),
    description='Say the same sentence in another way.',
    author='Jack Zhang',
    author_email='snail.zhang@outlook.com',
    url='https://www.zhangjack.com',
    license='MIT',
    install_requires=['jieba>=0.42.1', 'paddlepaddle-tiny==1.6.1', 'pypinyin>=0.37.0'],
    python_requires='>=3',
    package_data={'': ['*.txt']}
)
